/* ------------------------------------------------------------------------ */
/*                                                                          */
/*  Copyright (C)                                                           */
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 2 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful, but     */
/*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-    */
/*  ABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General        */
/*  Public License for more details.                                        */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   */
/*                                                                          */
/* ------------------------------------------------------------------------ */
/*  $Author: vogje02 $ $Revision: 1.3 $ $Date: 2002/07/28 17:13:19 $      */
/* ------------------------------------------------------------------------ */

#ifndef __LOG_WRITER_H
#define __LOG_WRITER_H

#include <afx.h>
#include <winbase.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#undef ERROR

// Log Levels
#define FATAL	    0
#define ERROR 	  1
#define WARNING   2
#define INFO	    3
#define DEBUG0    4
#define DEBUG1    5
#define DEBUG2    6

/**
 * Log writer.
 *
 * This class provides a commonly used log writer. Each line in the log file will have the same layout:\n
 * 
 * date - [level] - thread - class::method: format.\n
 *
 * The message can include a format statement. For instance:\n
 *
 * logWriter->log( DEBUG1, 123, "TestClass", "Method", "This is a log message from: %s", test);\n
 *
 * All C-like formats are accepted. The message is only logged when the log level is > than the debug
 * level. This means only message with level < debug_level will be written to the log file. This 
 * means also that starting the log writer with debug_level = 5 will write all message to the log file.
 * Note: This can be very noisy and will produce a lot of message written to the log file.
 *
 * The possible log levels are:\n
 *
 * FATAL: 0\n
 * ERROR: 1\n
 * WARNING: 2\n
 * INFO: 3\n
 * DEBUG0: 4\n
 * DEBUG1: 5\n
 *
 * In case the file name is NULL, all log messages will be written to stderr. Otherwise the file name
 * has to point to a valid file.
 *
 * One can set a socket to which to send the debug message. Use CLogWriter::SetSocket to write message
 * to the provided socket. At the moment only the CCP server is supporting this feature. Sending a special
 * debug message via CCP to the CCP server causes the CCP server to set a socket in its log writer and the
 * log writer will afterwards send all debug message to that socket.
 *
 * ccpdebug is using this feature.
 *
 * The log files are automatically rotated. The current log file has the name fileName. All other older
 * files have the name fileName.count (i.e. $JSPM_HOME\prf\log\prfagent.log is the current one and 
 * $JSPM_HOME\prf\log\prfagent.log.0 the previous one). The number of log files and the size can be
 * determined using 'count' and 'size'. The size has to be given in MBytes. Setting count=3 and
 * size=2 will produce 3 log files each of them 2MBytes.
 *
 * $Author: vogje02 $ $Revision: 1.3 $ $Date: 2002/07/28 17:13:19 $
 */
class CLogWriter
{
 public:

  /**
   * Constructor.
   *
   * In case the file name is NULL, all log messages will be written to stderr.
   *
   * @param file_name (char*) File name of the log file.
   * @param dl (int) Log level.
   */
  CLogWriter( char* file_name, int dl );

  /**
   * Constructor.
   *
   * In case the file name is NULL, all log messages will be written to stderr.
   *
   * @param file_name (char*) file name of the log file.
   * @param dl (int) log level.
   * @param count (int) number of log files for rotation.
   * @param size (int) size of each log file in MBytes
   */
  CLogWriter( char* file_name, int dl, int count, int size );

  /**
   * Destructor.
   */
  ~CLogWriter();

  /**
   * Initialization. 
   *
   * In case the file name is NULL, all log messages will be written to stderr.
   * 
   * @param file_name (char*) File name of the log file.
   * @param dl (int) Debug level.
   */
  void Init( char* file_name, int dl );

  /**
   * Writes a message to the log file
   * 
   * @param level (int) Log level.
   * @param cl (char*) Class name.
   * @param method (char*) Method name.
   * @param fmt (char*) Format of the message.
   */
  void log( int level, char* cl, char* method, char* fmt, ... );

  /**
   * Returns the file handel of the log file.
   * 
   * @return file (FILE*) File descriptor.
   */
  FILE *getFile();

 private:

  /**
   * Return the current local date/time
   * 
   * @param stime (char*) date/time in format day/month/year hour:min:sec:msec
   */
  void GetTime( char* stime );

  /**
   * Rotates the log files.
   *
   * Depending on the setting of 'count' and 'size' in the configuration file, this
   * method is rotating the log file after the size has been reached.
   */
  void Rotate();

  /**
   * Log file name
   */
  char fileName[256];

  /**
   * Log file
   */
  FILE *log_file;

  /**
   * Debug level.
   */
  int debug_level;

  /**
   * old_debug level.
   */
  int old_debug_level;

  /**
   * Size of the log file
   */
  long size;

  /**
   * Size of the log file
   */
  long maxSize;

  /**
   * Number of log files.
   */
  int logCount;

  /**
   * Current number of log files.
   */
  int currentLogCount;
};
#endif


